/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "game.h"

game_t GAME_STATE;

holdemgame_t holdem;
holdemrender_t render;

texture_t *fontTexture;
spritebatch_t *fontBatch;

bool gameInit() {
  // Init the game
  GAME_STATE.name = GAME_NAME;

  // Init the renderer.
  gameTimeInit();
  renderInit();
  inputInit();
  worldInit();

  holdemGameInit(&holdem);
  holdemRenderInit(&render);

  holdemRoundInit(&holdem);
  cardShuffle(holdem.deck, holdem.deckSize);

  // Deal Card
  holdemDealAll(&holdem, 2);
  holdemFlop(&holdem);

  fontTexture = assetTextureLoad("font.png");
  fontBatch = spriteBatchCreate(100);
  tileset_t *tileset = tilesetCreate(20, 20,
    fontTexture->width, fontTexture->height,
    1, 1, 1, 1
  );

  char *buffer = "!\"#$%";
  fontSpriteBatchBuffer(fontBatch, tileset, buffer, 0, 0, 0, 1.1, 1.5);

  // Load the world shader.
  GAME_STATE.shaderWorld = assetShaderLoad(
    "shaders/test.vert", "shaders/test.frag"
  );

  // Init the input manger.
  return true;
}

bool gameUpdate(float platformDelta) {
  gameTimeUpdate(platformDelta);
  renderFrameStart();
  inputUpdate();

  // Attach the world shader
  shaderUse(GAME_STATE.shaderWorld);

  // Set up the camera.
  int x = 10;
  cameraLookAt(&GAME_STATE.cameraWorld, x, 2.5f, 50, x, 2, 0);
  cameraPerspective(&GAME_STATE.cameraWorld, 45.0f,
    ((float)RENDER_STATE.width) / ((float)RENDER_STATE.height),
    0.5f, 500.0f
  );
  shaderUseCamera(GAME_STATE.shaderWorld, &GAME_STATE.cameraWorld);


  shaderUseTexture(GAME_STATE.shaderWorld, fontTexture);
  spriteBatchDraw(fontBatch, 0, -1);

  // holdemRender(&render, &holdem);
  // Render the game scene.
  // worldRender();

  if(inputIsPressed(INPUT_NULL)) return false;
  return true;
}

void gameDispose() {
  shaderDispose(GAME_STATE.shaderWorld);
  worldDispose();
  inputDispose();
  renderDispose();
}